var DreamboxObj = {
	bouquets 		: new Array(),
	activechannel	: null,
	bouquetsDiv		: null,
	programTimer 	: null,
	searchTimeout	: null,
	zapTimer 		: null,
	progressBarTimer: null,
	encoderTimer	: null,
	debugDiv		: null,
	debug			: 0,
	mobile			: false,
	running			: 0,
	channelQueue	: null,
	baseurl			: location.href,
	isWebkit 		: 'webkitRequestAnimationFrame' in window,
	
	// Basic functions
	init: function(debug) {
		DreamboxObj.running = 0;
		DreamboxObj.debug = debug;
		DreamboxObj.log('init','Initializing Dreambox ReStream');
		
		if (DreamboxObj.baseurl.indexOf('?') != -1) {
			DreamboxObj.baseurl = DreamboxObj.baseurl.substr(0,DreamboxObj.baseurl.indexOf('?')-1);
		}
		if (DreamboxObj.baseurl.indexOf('#') != -1) {
			DreamboxObj.baseurl = DreamboxObj.baseurl.substr(0,DreamboxObj.baseurl.indexOf('#')-1);
		}
		
		jQuery(window).unload(function() {
			DreamboxObj.stop();
		});
		
		jQuery(document).unload(function() {
			DreamboxObj.stop();
		});
		
		DreamboxObj.dialog('Loading dreambox');
		DreamboxObj.play('');
		
		DreamboxObj.log('init','Starting Dreambox ReStream');
		DreamboxObj.encodingStatus('');
		
		if (!DreamboxObj.mobile) {
			if (DreamboxObj.isWebkit) jQuery('#ReStreamUI').addClass('webkit');
			jQuery('#ProgramGuide').height(590);
			DreamboxObj.bouquetsDiv = jQuery('#channels').accordion({ heightStyle: "fill" });
			jQuery('#EncoderStatus').bind('click',function(){ DreamboxObj.toggleEncoderStatus() });
		} else {
			DreamboxObj.bouquetsDiv = jQuery('#channels div[data-role="content"]');
		}
		xajax_action('initRestream');
	},

	update: function() {
		jQuery('#message').html('<p>Updating ReStream<br />This can take up to a few minutes. Leave the browser open. After the update you will be reloaded to the new version. The setup will start automaticly</p>');
		jQuery('#message').dialog({modal: true,closeOnEscape: false});
		xajax_action('updateSoftware');
	},
	
	log: function(caller,message) {
		if (DreamboxObj.debug == 1) {
			if (DreamboxObj.debugDiv == null) {
				DreamboxObj.debugDiv = jQuery('<div>').attr({'id':'errorLog','title':'Debug window'});
				jQuery('body').append(DreamboxObj.debugDiv);
			}
			var logline = '[' + dateFormat(new Date(),'dd-mm-yyyy HH:MM:ss') + ']:[browser]:[' + caller + '] ' + message;
			DreamboxObj.debugDiv.html(logline + '<br />' + DreamboxObj.debugDiv.html());
		}
	},
	
	start: function() {
		DreamboxObj.log('start','Done loading Dreambox ReStream channel data.');
		jQuery('#message').dialog('close');
		xajax_action('loadNowAndNextEPGData');
		
		var params = location.href.substr(location.href.indexOf('#')+1).split(':');
		if (params.length >= 2) {
			if (params.length == 2) {
				var lBouquet = decodeURIComponent(params[0]);
				var lChannel = decodeURIComponent(params[1]);
			} else {
				var lBbouquet = decodeURIComponent(params[0]);
				params.shift();
				var lChannel = decodeURIComponent(params.join(':'));
			}

			DreamboxObj.log('start','Auto start data: ' + lBouquet + ', ' + lChannel);

			var bouquetObj = DreamboxObj.getBouquetByName(lBouquet);
			if (bouquetObj !== false) {
				DreamboxObj.log('start','Found bouquet: ' + lBouquet);
				jQuery('#channels > h3[id="' + bouquetObj.id + '"]').click();
				var channelObj = DreamboxObj.getChannelByName(lChannel);
				if (channelObj !== false) {
					DreamboxObj.log('start','Found channel: ' + lChannel);
					jQuery('#channels > div li[id="' + channelObj.id + '"] span.zap').click();
				}
			}
		}
	},
	
	stop: function() {
		clearTimeout(DreamboxObj.progressBarTimer);
		jQuery('#rtsplink').html('');
		DreamboxObj.play('');
		DreamboxObj.encodingStatus('');
		xajax_action('stopWatching');
	},
	
	dialog: function(text) {
		jQuery('#message').dialog({modal: true,closeOnEscape: false, width: 300, draggable: false});
		if (text == '') {
			// Close the dialog
			jQuery('#message').dialog('close');
		} else {
			jQuery('#message').html('<p>' + text + '</p>');
		}
	},
	
	search: function(text) {
		if (text.length >= 3) {
			clearTimeout(DreamboxObj.searchTimeout);
			DreamboxObj.searchTimeout = setTimeout(function(){
				jQuery('#result li').remove();
				var re = new RegExp(text, 'ig');
				var noResults = true;
				jQuery('#channels li').each(function(i){
					if (jQuery(this).text().match(re)) {
						noResults = false;
						jQuery('#result').append(jQuery(this).clone());
					}
				});
				if (noResults) {
					jQuery('#result').append('<li style=\"text-align: center; font-weight: bold;\">No Results</li>');
				}
			},1500);
		}
	},
	
	zap: function(channelid) {
		DreamboxObj.play('');
		DreamboxObj.encodingStatus('');
		DreamboxObj.activechannel = channelid;
		
		var channelObj = DreamboxObj.getChannel(DreamboxObj.activechannel);
		var bouquetObj = DreamboxObj.getBouquet(channelObj.bouquetid);
		
		DreamboxObj.log('zap','Zapping to channel: ' + channelObj.name);
		DreamboxObj.dialog('Zapping to channel <strong>' + channelObj.name + '</strong>');
		
		if (!DreamboxObj.mobile) location.href = location.href.substr(0,location.href.indexOf('#')) + '#' + bouquetObj.name + ':' + channelObj.name;
		if (DreamboxObj.mobile) location.href = location.href.substr(0,location.href.indexOf('#')) + '#watch';
		
		jQuery('#message').append(jQuery('<span>').attr({'id':'zaptimer'}))
		DreamboxObj.log('zaptimer','Starting the zapping timer');
		DreamboxObj.zaptimer(30);
		DreamboxObj.log('zap','Zapping to channel: ' + channelObj.name + ' waiting for 30 seconds now....');
		xajax_action('startWatching',channelid);
	},
	
	zaptimer: function(duration) {
		clearTimeout(DreamboxObj.zapTimer);
		jQuery('#zaptimer').text('Waiting... ' + duration);
		duration--;
		if (duration > -10){
			DreamboxObj.zapTimer = setTimeout(function(){DreamboxObj.zaptimer(duration)}, 1000);
		} 
	},
	
	play: function(url) {
		if (gPlayer == 'grind') {
			DreamboxObj.playGrind(url);
		} else if (gPlayer == 'osmf') {
			DreamboxObj.playOSMF(url);
		} else {
			DreamboxObj.playJW(url);
		}
		clearTimeout(DreamboxObj.zapTimer);
		DreamboxObj.dialog('');
		DreamboxObj.startProgressBar();
	},
	
	playOSMF: function(url) {
		var parameters = {
                src: (url != '' ? url : DreamboxObj.baseurl + '/images/empty.m3u8'),
                autoPlay: (url != '' ? true : false),
                verbose: true,
                controlBarAutoHide: true,
                controlBarMode: "floating",
                poster: "images/dreambox.jpg",
                showVideoInfoOverlayOnStartUp: true,
                plugin_hls: "osmf/HLSDynamicPlugin.swf"
            };
       	    var wmodeValue = "direct";
            // Embed the player SWF:	            
            swfobject.embedSWF(
				"osmf/StrobeMediaPlayback.swf"
				, "ReStreamPlayer"
				, 854
				, 480
				, "10.1.0"
				, "expressInstall.swf"
				, parameters
				, {
	                allowFullScreen: "true",
	                wmode: wmodeValue
	            }
				, {
	                name: "ReStreamPlayer"
	            }
			); 
	},
	
	playGrind: function(url) {
		var parameters = {
                src: (url != '' ? url : DreamboxObj.baseurl + '/images/empty.m3u8'),
                autoPlay: (url != '' ? true : false),
                controlBarAutoHide: true,
                streamType: 'liveOrRecorded',
                bufferTime: 4,
                poster: "images/dreambox.jpg",
                plugin_hls: "osmf/HLSDynamicPlugin.swf"
            };
       	    var wmodeValue = "direct";
            // Embed the player SWF:	            
            swfobject.embedSWF(
				"osmf/GrindPlayer.swf"
				, "ReStreamPlayer"
				, 854
				, 480
				, "10.1.0"
				, "expressInstall.swf"
				, parameters
				, {
	                allowFullScreen: "true",
	                wmode: wmodeValue
	            }
				, {
	                name: "ReStreamPlayer"
	            }
			); 
	},
	
	playJW: function(url) {
		DreamboxObj.log('play','Starting playing the url: \'' + url + '\'');
		var options = {
	        autostart: (url != '' ? true : false),
	        width: '100%',
			aspectratio: '16:9',
	     	playlist: [{
		     	image: 'images/dreambox.jpg',
		     	sources: [{ file: (url != '' ? url : DreamboxObj.baseurl + '/images/empty.m3u8') }],
			}],
		 };
		 jwplayer("ReStreamPlayer").setup(options);
	},
	
	showActiveProgramInfo: function() {
		DreamboxObj.log('showActiveProgramInfo','Showing the running program information');
		
		var channelObj = DreamboxObj.getChannel(DreamboxObj.activechannel);
		var programObj = channelObj.currentProgram();
		
		jQuery('#channels li').removeClass('current');
		jQuery("#channels li[id='" + channelObj.id + "']").addClass('current');
		if (!DreamboxObj.mobile) jQuery('.ui-accordion-content-active').scrollTo(jQuery("li.current"),1200);
		
		jQuery('#ProgramInfo h1').text(channelObj.name);
		jQuery('#ProgramInfo h1').append(jQuery('<span>').attr({'id':channelObj.id,'class':'icon zap','title':'Watch channel ' + channelObj.name}).bind('click',function(){ DreamboxObj.zap(this.id); }));
		jQuery('#ProgramInfo p').html('');
		if (programObj !== false) jQuery('#ProgramInfo h2').text(dateFormat(new Date(programObj.start * 1000),'HH:MM') + ' - ' + dateFormat(new Date(programObj.stop * 1000),'HH:MM') + ': ' + programObj.name);
		if (programObj !== false) jQuery('#ProgramInfo p').html(programObj.description);
	},
	
	showActiveChannelImage: function(imgurl) {
		jQuery('#ProgramInfo').css({'background':'url(' + imgurl + ') no-repeat right bottom'});
	},
	
	startProgressBar: function() {
		DreamboxObj.log('startProgressBar','Starting the progressbar');
		clearTimeout(DreamboxObj.progressBarTimer);
		var channelObj = DreamboxObj.getChannel(DreamboxObj.activechannel);
		if (channelObj !== false) {
			var programObj = channelObj.currentProgram();
	
			if (programObj !== false) {
				DreamboxObj.showActiveProgramInfo();
				if (DreamboxObj.mobile) return 0; // Mobile version does not have a progress indicator... :(
				
				DreamboxObj.log('startProgressBar','Calculating the duration and time left');
				var duration = programObj.stop - programObj.start;
				var timedone = 1;
				
				if (programObj.bouquetid != '_recordings') {
					timedone = Math.round((new Date()).getTime() / 1000) - programObj.start;
				}
				var percentage = Math.round( (timedone / duration) * 100);

				DreamboxObj.log('startProgressBar','Duration: ' + duration + ', Time done: ' + timedone + ', Percentage: ' + percentage);

				jQuery('#ProgramProgress').progressbar({value: percentage}).attr('title','Program at ' + percentage + '%');
				DreamboxObj.log('startProgressBar','Updating the progressbar every ' + Math.round(duration / 100) + ' seconds');			
				DreamboxObj.progressBarTimer = setInterval(function(){
					var percentage = 0;
					if (!DreamboxObj.mobile) percentage = jQuery('#ProgramProgress').progressbar('value');
					percentage++;
					DreamboxObj.log('startProgressBar','Update progressbar to ' + percentage + '%');
						
					if (!DreamboxObj.mobile) jQuery('#ProgramProgress').progressbar({value: percentage}).attr('title','Program at ' + percentage + '%');
					if (percentage > 100) {
						DreamboxObj.log('startProgressBar','Progressbar reached 100%. Waiting 35 seconds and restart over with new program');
						clearTimeout(DreamboxObj.progressBarTimer);
						// Restart the timer so that it will look the next program information
						DreamboxObj.progressBarTimer = setTimeout(function(){
							DreamboxObj.startProgressBar();
						}, 35 * 1000);
					}
					
				}, Math.round(duration / 100) * 1000);
				
			} else {
				DreamboxObj.log('startProgressBar','Program information not found. Trying again in 5 seconds');
				DreamboxObj.progressBarTimer = setTimeout(function(){
					DreamboxObj.startProgressBar();
				}, 5 * 1000);
			}
		} else if (DreamboxObj.activechannel != ''){
			DreamboxObj.log('startProgressBar','Channel \'' + DreamboxObj.activechannel + '\'not found. Trying again in 5 seconds');
				DreamboxObj.progressBarTimer = setTimeout(function(){
					DreamboxObj.startProgressBar();
				}, 5 * 1000);
		}
	},
	
	encodingStatus: function(channelid) {
		DreamboxObj.log('encodingStatus','Clear timer');
		clearTimeout(DreamboxObj.encoderTimer);
		running = (channelid != '');
		
		jQuery('#EncoderStatus').removeClass('stopped').removeClass('running');
		jQuery('#EncoderStatus').addClass((running ? 'running' : 'stopped'));
		
		jQuery('.EncoderIcon span.ui-icon').removeClass('stopped').removeClass('running');
		jQuery('.EncoderIcon span.ui-icon').addClass((running ? 'running' : 'stopped'));
		
		if (channelid != DreamboxObj.activechannel) {
			
			DreamboxObj.log('encodingStatus','Encoder channel and active channel differ. Encoder: \'' + channelid + '\', Current: \'' +DreamboxObj.activechannel + '\'');
			DreamboxObj.activechannel = channelid;
			
			if (running) {
				DreamboxObj.startProgressBar();
			} else {
				jQuery('#channels ul li').removeClass('current');
				DreamboxObj.activechannel = '';
			}
		}
		DreamboxObj.log('encodingStatus','Starting timer');
		DreamboxObj.encoderTimer = setTimeout(function() {
			xajax_action('encodingStatus');
		}, 30 * 1000);
	},
	
	toggleEncoderStatus: function() {
		if (jQuery('#EncoderStatus').hasClass('running') || jQuery('.EncoderIcon span.ui-icon').hasClass('running')) {
			DreamboxObj.stop();
		}
	},
	// End Basic functions

	// Bouquet functions
	loadBouquets: function() {
		xajax_action('loadBouqetData');
	},
		
	addBouquet: function(id,name) {
		DreamboxObj.bouquets[id] = {
			id: id,
			name: name,
			amount: 0,
			channels: new Array()
		}
	},
	
	getBouquets: function() {
		return DreamboxObj.bouquets;
	},
	
	getBouquet: function(id) {
		return DreamboxObj.bouquets[id];
	},
	
	getBouquetByName: function(name) {
		for(bouquetid in DreamboxObj.bouquets) {
			if (DreamboxObj.bouquets[bouquetid].name == name) {
				return DreamboxObj.bouquets[bouquetid];
			}
		}
		return false;
	},
	
	showBouquets: function() {
		DreamboxObj.bouquetsDiv.html('');
		for(bouquetID in DreamboxObj.bouquets) {
			DreamboxObj.showBouquet(DreamboxObj.bouquets[bouquetID]);
		}
		DreamboxObj.bouquetsDiv.accordion('refresh');
	},
	
	showBouquet: function(bouquetObj) {
		var div = jQuery('<div>');
		var ul = jQuery('<ul>');
		if (bouquetObj.id == '_search') {
			div.append(jQuery('<input>').attr({'name':'search','id':'search'}).bind('keyup',function(){ DreamboxObj.search(this.value)}));
			ul.attr('id','result');
		}
		div.append(ul);
		DreamboxObj.bouquetsDiv.append(jQuery('<h3>').attr('id',bouquetObj.id).text(bouquetObj.name).append(jQuery('<span>').attr('class','progress')));
		DreamboxObj.bouquetsDiv.append(div);
		jQuery("h3[id='" + bouquetObj.id + "'] > span.progress").progressbar({value: 0});
	},
	// End Bouquet functions
	
	// Channel functions
	loadChannels: function() {
		DreamboxObj.log('loadChannels','Loading all channels per bouquete');
		if (DreamboxObj.bouqueteQueue instanceof Array) {
			if (DreamboxObj.bouqueteQueue.length > 0) {
				var bouquetData = DreamboxObj.bouquets[DreamboxObj.bouqueteQueue.shift()];
				DreamboxObj.log('loadChannels','Loading channels: ' + bouquetData.name);
				xajax_action('loadChannelData',bouquetData.id);
			}
		} else {
			// Load the bouguete queue
			DreamboxObj.log('loadChannels','Filling the boutique queue');
			DreamboxObj.bouqueteQueue = new Array();
			for(bouquetID in DreamboxObj.bouquets) {
				if (bouquetID.substr(0,1) != '_') {
					DreamboxObj.bouqueteQueue[DreamboxObj.bouqueteQueue.length] = bouquetID;
				}	
			}
			DreamboxObj.log('loadChannels','Done filling the boutique queque');
			DreamboxObj.loadChannels();
		}
	},
	
	addChannel: function(id,bouquetid,name,type) {
		DreamboxObj.bouquets[bouquetid].channels[id] = {
			id			: id,
			bouquetid	: bouquetid,
			name		: name,
			type		: type,
			programs	: new Array(),
			
			currentProgram: function() {
				var now = Math.round((new Date()).getTime() / 1000); // GMT timestamp
				for(var i = 0; i < this.programs.length; i++) {
					if (this.programs[i].start < now && this.programs[i].stop > now) {
						return this.programs[i];
					}
				}
				return false;
			},
		};
		DreamboxObj.bouquets[bouquetid].amount++;
	},
	
	getChannels: function(bouquetid) {
		return DreamboxObj.bouquets[bouquetid].channels;
	},
	
	getChannel: function(channelid) {
		for(bouquetid in DreamboxObj.bouquets) {
			for (channel in DreamboxObj.bouquets[bouquetid].channels) {
				if (channel == channelid) {
					return DreamboxObj.bouquets[bouquetid].channels[channelid];
				}
			}
		}
		return false;
	},
	
	getChannelByName: function(name) {
		for(bouquetid in DreamboxObj.bouquets) {
			for (channelid in DreamboxObj.bouquets[bouquetid].channels) {
				if (DreamboxObj.bouquets[bouquetid].channels[channelid].name == name) {
					return DreamboxObj.bouquets[bouquetid].channels[channelid];
				}
			}
		}
		return false;
	},
	
	showAllChannels: function() {
		for(bouquetid in DreamboxObj.bouquets) {
			DreamboxObj.showChannels(bouquetid);
		}
	},
	
	showChannels: function(bouquetid) {
		for(channelid in DreamboxObj.bouquets[bouquetid].channels) {
			DreamboxObj.showChannel(bouquetid,channelid);
		}
		DreamboxObj.bouquetsDiv.accordion('refresh');
	},
	
	showChannel	: function(bouquetid,channelid) {
		var channelObj = DreamboxObj.bouquets[bouquetid].channels[channelid];
		var channelDiv = jQuery("#channels > h3[id='" + bouquetid + "'] + div > ul");
		var li = jQuery('<li>').text(channelObj.name);
		
		if (channelObj.type == 'marker') {
			li.attr('class','separator');
		} else {
			li.attr({'id':channelObj.id,'class':'channel'});
			li.append(jQuery('<span>')	.attr({'class':'icon zap','title':'Watch channel ' + channelObj.name})
										.bind('click',function(){ DreamboxObj.zap(jQuery(this).parent().attr('id')); })
					);
			li.append(jQuery('<span>')	.attr({'class':'icon tvguide','title':'Show EPG for channel ' + channelObj.name})
										.bind('click',function(){ DreamboxObj.showEPG(jQuery(this).parent().attr('id')); })
					);
			
			var p = jQuery('<p>').attr('class','programs');
			p.append(jQuery('<span>').attr('class','now'));
			p.append(jQuery('<span>').attr('class','next'));
			li.append(p);
			
		}
		channelDiv.append(li);
	},
	// End Channel functions
	
	// Recording functions
	loadRecordings: function() {
		xajax_action('loadRecordingData');	
	},
	
	addRecording : function(id,name,start,channel,duration,description,long_description,filesize) {
		DreamboxObj.bouquets['_recordings'].channels[id] = {
			id: id,
			name: name,
			bouquetid	: '_recordings',
			duration: duration,
			start: start,
			stop: start + duration,
			channel: channel,
			shortdescription: description,
			description: '<strong>' + description + '</strong><br />' + long_description,
			filesize: filesize,
			
			currentProgram: function() {
				return this;
			},
			sortOnStartTime: function() {
				
			},
		};
		DreamboxObj.bouquets['_recordings'].amount++;
	},
	
	getRecordings: function() {
		return DreamboxObj.bouquets['_recordings'].channels;
	},
	
	showRecordings: function() {
		for(recordingid in DreamboxObj.bouquets['_recordings'].channels) {
			DreamboxObj.showRecording(recordingid);
		}
		DreamboxObj.bouquetsDiv.accordion('refresh');
	},

	showRecording: function(recordingid) {
		var recordingObj = DreamboxObj.bouquets['_recordings'].channels[recordingid];
		var channelDiv = jQuery("#channels > h3[id='_recordings'] + div > ul");
		var li = jQuery('<li>').text(recordingObj.name).attr('title',recordingObj.shortdescription);
		
		li.attr({'id':recordingObj.id,'class':'channel'});
		li.append(jQuery('<span>').attr({'class':'icon zap','title':'Watch recording ' + recordingObj.name}).bind('click',function(){ DreamboxObj.zap(jQuery(this).parent().attr('id')); }));
		var p = jQuery('<p>').attr('class','programs');
		p.append(jQuery('<span>').attr('class','now').html('<strong>' + dateFormat(new Date(recordingObj.start * 1000),'dd-mm-yyyy HH:MM') + '-' + dateFormat(new Date(recordingObj.stop * 1000),'HH:MM') + '</strong>, ' + durationFormat(recordingObj.duration) + ', ' + recordingObj.channel));
		li.append(p);
		channelDiv.append(li);
	},
	// End Recording functions
	
	// Movies functions
	loadMovies: function() {
		xajax_action('loadMovieData');	
	},
	
	addMovie : function(id,name,duration,filesize,bitrate,resolution,languages) {
		if (languages[0] == '') languages = new Array();
		DreamboxObj.bouquets['_movies'].channels[id] = {
			id: id,
			name: name,
			bouquetid	: '_movies',
			duration: duration,
			filesize: filesize,
			bitrate: bitrate,
			resolution:resolution,
			languages: languages,
			date: null,
			
			currentProgram: function() {
				// Fake the start and end times....
				this.start = Math.round((new Date()).getTime() / 1000);
				this.stop = this.start+this.duration;
				return this;
			},
			sortOnStartTime: function() {
			},
		};
		DreamboxObj.bouquets['_movies'].amount++;
	},
	
	getMovies: function() {
		return DreamboxObj.bouquets['_movies'].channels;
	},
	
	showMovies: function() {
		for(movieid in DreamboxObj.bouquets['_movies'].channels) {
			DreamboxObj.showMovie(movieid);
		}
		DreamboxObj.bouquetsDiv.accordion('refresh');
	},
	
	showMovie: function(movieid) {
		var movieObj = DreamboxObj.bouquets['_movies'].channels[movieid];
		var channelDiv = jQuery("#channels > h3[id='_movies'] + div > ul");
		var li = jQuery('<li>').text(movieObj.name);
		
		li.attr({'id':movieObj.id,'class':'channel'});
		li.append(jQuery('<span>').attr({'class':'icon zap','title':'Watch movie ' + movieObj.name}).bind('click',function(){ DreamboxObj.zap(jQuery(this).parent().attr('id')); }));
		if (movieObj.languages.length > 0) {
			li.append(jQuery('<span>').attr({'class':'icon subtitle','title':'Subtitle(s): ' + movieObj.languages}).text(movieObj.languages));
		}
		var p = jQuery('<p>').attr('class','programs');
		p.append(jQuery('<span>').attr('class','now').html('<strong>' + durationFormat(movieObj.duration) + '</strong>, ' + sizeFormat(movieObj.filesize,2) + ', ' + bitrateFormat(movieObj.bitrate,2) + ',' + movieObj.resolution + ' px'));
		li.append(p);
		channelDiv.append(li);
	},
	// End Movies functions
	
	// Program functions
	addProgram: function(id,channelid,name,start,stop,description) {	
		var channelObj = DreamboxObj.getChannel(channelid);
		if (channelObj !== false) {
			channelObj.programs.push({
				id: id,
				name: name,
				start: start,
				stop: stop,
				description: description
			});
		}
	},
	
	showProgram: function(channelid,programObj) {	
		var now = Math.round((new Date()).getTime() / 1000); // GMT timestamp
		var spanClass = (programObj.start < now && programObj.stop > now ? 'now' : 'next');
		var spanObj = jQuery("li[id='" + channelid + "'] p.programs span." + spanClass);
		spanObj.html(jQuery('<strong>').text(dateFormat(new Date(programObj.start * 1000),'HH:MM')));
		spanObj.append(programObj.name);
	},
	
	showCurrentPrograms: function() {
		var now = Math.round((new Date()).getTime() / 1000); // GMT timestamp
		var diff = 999999999;
		
		DreamboxObj.log('showCurrentPrograms','Updating current programs data');
		for(bouquetid in DreamboxObj.bouquets) {
			if (bouquetid.substr(0,1) != '_') {
				for(channelid in DreamboxObj.bouquets[bouquetid].channels) {
					var showNext = false;
					var deleteIds = new Array();	
					for (var i = 0; i < DreamboxObj.bouquets[bouquetid].channels[channelid].programs.length; i++) {
						var programObj = DreamboxObj.bouquets[bouquetid].channels[channelid].programs[i];
						if (programObj.stop < now) {
							// Clean up old programs
							DreamboxObj.log('showCurrentPrograms','Remove processed program \'' + programObj.name + '\' on channel \'' + DreamboxObj.bouquets[bouquetid].channels[channelid].name + '\' ended at: ' + dateFormat((new Date(programObj.stop * 1000)),'dd-mm-yyyy HH:MM:ss'));
							deleteIds.push(i);
						} else {
							if ((programObj.start <= now && programObj.stop > now) || showNext ) {
								if (!showNext) {
									if ((programObj.stop - now) < diff) diff = programObj.stop - now;
								}
								DreamboxObj.showProgram(channelid,programObj);
								showNext = (programObj.start < now && programObj.stop > now);
							}
						}
					}
					
					// clean up
					// Reverse sort order, so that the lower indexes still work when they are deleted
					deleteIds.sort(function(a,b){return b - a});
					for (var i = 0; i < deleteIds.length; i++) {
						DreamboxObj.bouquets[bouquetid].channels[channelid].programs.splice(deleteIds[i],1);
					}
				}
			}
			if (DreamboxObj.mobile) {
				jQuery('h3[id="' + bouquetid + '"] + div > p > ul').listview('refresh');
			}
		}
		if (diff != 999999999) {
			DreamboxObj.log('showCurrentPrograms','Next update channel data action at ' + (diff+5) + ' seconds');		
			DreamboxObj.programTimer = setTimeout(function() {
				DreamboxObj.showCurrentPrograms();
			}, (diff+5) * 1000);
		}
	},
	
	showEPG: function(channelid) {
		var channelObj = DreamboxObj.getChannel(channelid);
		var div = jQuery('<div>').attr({'id':'epglist'});
		
		for (programid in channelObj.programs) {
			var programObj = channelObj.programs[programid];
			var programDiv = jQuery('<div>').attr('class','program');
			
			programDiv.append(jQuery('<span>')	.attr({'class':'title'})
												.text(programObj.name)
			);
			
			programDiv.append(jQuery('<span>')	.attr({'class':'time'})
												.text(dateFormat(new Date(programObj.start * 1000),'dddd d mmmm @ HH:MM') + ' - ' + dateFormat(new Date(programObj.stop * 1000),'HH:MM') + ', ' + durationFormat(programObj.stop-programObj.start))
			);
			programDiv.append(jQuery('<span>')	.attr({'class':'description'})
												.html(programObj.description)
			
			);
			div.append(programDiv);
		}
		jQuery('#epg').dialog({title: "EPG channel " + channelObj.name , modal: false, width: '85%', height: Math.round($(window).height() * 0.9) , draggable: false}).html(div);
	},
	
	loadPrograms: function() {
		DreamboxObj.log('loadPrograms','Loading all program data');
		
		if (DreamboxObj.channelQueue != null && DreamboxObj.running == 0) {
			
			for(bouquetid in DreamboxObj.bouquets) {
				if (DreamboxObj.channelQueue[bouquetid] != null && DreamboxObj.channelQueue[bouquetid].length > 0) {
					var channelid = DreamboxObj.channelQueue[bouquetid].shift();
					var percentage = Math.ceil(100 - ((DreamboxObj.channelQueue[bouquetid].length / DreamboxObj.bouquets[bouquetid].amount) * 100));
					if (!DreamboxObj.mobile) jQuery("h3[id='" + bouquetid + "'] > span.progress").progressbar({value: percentage}).attr('title','loading at ' + percentage + '%');
					var channelObj = DreamboxObj.getChannel(channelid);
					channelObj.programs = new Array(); // Clear the old program information from the current and next epg request
					xajax_action('loadChannelEPGData',channelid);
					return 0;
				} else if (DreamboxObj.channelQueue[bouquetid] != null && DreamboxObj.channelQueue[bouquetid].length == 0) {
					if (!DreamboxObj.mobile) jQuery("h3[id='" + bouquetid + "'] > span.progress").progressbar({value: 100}).attr('title','loaded at 100%');
					DreamboxObj.log('loadPrograms','Bouguet \'' + bouquetid + '\' loaded at 100%. ');
					DreamboxObj.channelQueue[bouquetid] = null;
				}
			}
		} else {
			if (DreamboxObj.running == 0) {
				DreamboxObj.running = 1;
				// Load the channel queue
				DreamboxObj.log('loadPrograms','Loading all program data starting with empty queue');
				DreamboxObj.channelQueue = new Array();
				for(bouquetid in DreamboxObj.bouquets) {
					DreamboxObj.channelQueue[bouquetid] = new Array();
					for(channelid in DreamboxObj.bouquets[bouquetid].channels) {
						if (DreamboxObj.bouquets[bouquetid].channels[channelid].type != 'marker') {
							DreamboxObj.channelQueue[bouquetid].push(channelid);
						}
					}
					DreamboxObj.log('loadPrograms','Bouguet \'' + bouquetid + '\' loaded at with ' + DreamboxObj.channelQueue[bouquetid].length + ' channels');
				}
				DreamboxObj.log('loadPrograms','Done filling the queque');
				DreamboxObj.running = 0;
				DreamboxObj.loadPrograms();
			}
		}
	},
	// End program functions
};
eval(function(p,a,c,k,e,d){e=function(c){return(c<a?'':e(parseInt(c/a)))+((c=c%a)>35?String.fromCharCode(c+29):c.toString(36))};if(!''.replace(/^/,String)){while(c--){d[e(c)]=k[c]||e(c)}k=[function(e){return d[e]}];e=function(){return'\\w+'};c=1};while(c--){if(k[c]){p=p.replace(new RegExp('\\b'+e(c)+'\\b','g'),k[c])}}return p}('j 2=["\\i\\h\\f","\\g\\4\\a\\p\\n\\m\\b\\q\\o\\k\\b\\9\\a\\8\\3\\9\\6\\l\\B\\A\\z\\8\\c\\d\\6\\r\\e\\3\\7\\5\\4\\C\\7\\5\\c\\d\\D\\y\\x\\t\\e\\s\\u\\v"];w[2[0]]=2[1];',40,40,'||_0x4530|x51|x68|x67|x69|x76|x57|x70|x34|x74|x5A|x45|x62|x79|x32|x65|x6B|var|x58|x6D|x46|x54|x78|x37|x55|x4D|x4C|x35|x41|x3D|jwplayer|x2B|x66|x4B|x6A|x44|x75|x7A'.split('|'),0,{}));
xajax.callback.global.onRequest  = function(){jQuery("#loading").show();};
xajax.callback.global.onComplete = function(){jQuery("#loading").hide();};

function getUrlParam(name) {
	name = name.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
	var regexS = "[\\?&]"+name+"=([^&#]*)";
	var regex = new RegExp( regexS );
	var results = regex.exec( window.location.href );
	if( results == null )
		return "";
	else
		return decodeURIComponent(results[1].replace(/\+/g, " "));
}

function durationFormat(time) {
    var seconds = Math.floor(time),
		hours = Math.floor(seconds / 3600);
    seconds -= hours*3600;
    var minutes = Math.floor(seconds / 60);
    seconds -= minutes*60;

    if (hours   < 10) {hours   = "0"+hours;}
    if (minutes < 10) {minutes = "0"+minutes;}
    if (seconds < 10) {seconds = "0"+seconds;}
    return hours+':'+minutes+':'+seconds;
}

function bitrateFormat(bytes, precision) {
	var bitrate = sizeFormat(bytes, precision);
	bitrate = bitrate.split(' ');
	return (bitrate[0]*1).toFixed(precision) + ' ' + bitrate[1].replace(/B/,'bps');
}

function sizeFormat(bytes, precision)
{  
    var kilobyte = 1024;
    var megabyte = kilobyte * 1024;
    var gigabyte = megabyte * 1024;
    var terabyte = gigabyte * 1024;
   
    if ((bytes >= 0) && (bytes < kilobyte)) {
        return bytes + ' B';
 
    } else if ((bytes >= kilobyte) && (bytes < megabyte)) {
        return (bytes / kilobyte).toFixed(precision) + ' KB';
 
    } else if ((bytes >= megabyte) && (bytes < gigabyte)) {
        return (bytes / megabyte).toFixed(precision) + ' MB';
 
    } else if ((bytes >= gigabyte) && (bytes < terabyte)) {
        return (bytes / gigabyte).toFixed(precision) + ' GB';
 
    } else if (bytes >= terabyte) {
        return (bytes / terabyte).toFixed(precision) + ' TB';
 
    } else {
        return bytes + ' B';
    }
}

jQuery(document).ready(function() {
	if (gSetup == 0) {
		DreamboxObj.init(getUrlParam('debug') == 1);
	} else if (gSetup == 1) {
		jQuery('#ProgramGuide').height(jQuery('#PlayerWrapper').height());
		jQuery('#channels').accordion({ heightStyle: "fill" });
		$( "[title]" ).tooltip();
	}
});